;; ==========================================================
;; DITCH model version 1.0
;; ----------------------------------------------------------
;; Author:              Ruth Meyer
;; Model specification: Laurence Lessard-Phillips, Huw Vasey
;; ==========================================================

extensions [ 
  array
  pathdir 
]

globals [
  start-x-positions start-y-positions
  gender g-shape
  ethnicity e-colour e-proportions
  age-distribution-bounds age-distribution-probs
  Schelling-threshold
  marriage-counts dates-before-marriage 
  traced-turtles
  file-output? sep output-dir
  education-proportions-men education-proportions-women education-levels ethnic-groups
  dropped-links dates-level marriages-level
  ;; for output measures
  num-turtles ;; array of number of turtles per ethnicity
  married-turtles ;; array of number of married turtles per ethnicity
  out-percent ;; array of percent of cross-ethnic marriages per ethnicity
  cross-ethnic ;; percent of cross-ethnic marriages overall
]

turtles-own [
  traits
  preferences
  p-candidate
  satisfied?
  already-tried?
  partner
  marriage-tick
  recently-dated
  date-counter
  dating-tick
  dating-duration
  date-level
]

to setup 
  clear-all
  show "**************** new model run ***************"
  show ""
  
  ;; set global variables
  set start-x-positions (list (min-pxcor / 2) (max-pxcor / 2) (min-pxcor / 2)  (max-pxcor / 2) 0)  
  set start-y-positions (list (min-pycor / 2) (max-pycor / 2) (max-pycor / 2)  (min-pycor / 2) 0)
  set gender (list "M" "F")
  set g-shape (list "male" "female") 
  set ethnicity (list "w" "x" "y" "z")
  set e-colour (list orange green yellow cyan)
  set e-proportions read-from-string eth-proportions 
  set age-distribution-bounds [20 25 30 35 36]
  set age-distribution-probs [0.0999 0.2498 0.2748 0.3129 0.0626]  ;; values are from 2001 census
  set Schelling-threshold 0.35
  set marriage-counts init-marriage-counts
  set dates-before-marriage [0 0]  ;; first element accumulates number of dates, second element accumulates number of married people
  set traced-turtles []
  set file-output? false
  set dropped-links 0
  set dates-level n-values love-radar [0] ;; counts the number of dates with partners of level 1 (direct links), level 2 (friends of friends), level 3...
  set marriages-level n-values love-radar [0] ;; counts the number of marriages with partners of level 1 (direct links), level 2 (friends of friends), level 3...
  set ethnic-groups ["White British" "White Irish" "Other White" 
                     "White and Black Carib" "White and Black Afric" "White and Asian" "Other Mixed" 
                     "Indian" "Pakistani" "Bangladeshi" "Other Asian" 
                     "Black Caribbean" "Black African" "Other Black" 
                     "Chinese" "Other Ethnic Group"]
  set education-levels ["none" "level1" "level2" "level3" "level4/5"]
  set education-proportions-men [
    [0.1545 0.2559 0.2362 0.1129 0.2406]
    [0.1482 0.134 0.1708 0.1478 0.3992]
    [0.1343 0.0716 0.1157 0.1569 0.5215]
    [0.1925 0.2821 0.2312 0.1017 0.1925]
    [0.1739 0.1546 0.1981 0.1425 0.3309]
    [0.1411 0.1388 0.1682 0.1716 0.3804]
    [0.145 0.1341 0.1829 0.1721 0.3659]
    [0.1542 0.1398 0.1428 0.1332 0.43]
    [0.3394 0.1484 0.1253 0.1089 0.278]
    [0.4283 0.1511 0.1243 0.0872 0.2091]
    [0.1746 0.1596 0.1618 0.1516 0.3524]
    [0.1762 0.2971 0.2501 0.0828 0.1938]
    [0.0849 0.0883 0.1869 0.1382 0.5017]
    [0.1782 0.2318 0.251 0.1188 0.2203]
    [0.1514 0.0679 0.0916 0.1763 0.5128]
    [0.1926 0.0676 0.0833 0.1332 0.5232]
  ] 
  set education-proportions-women [
    [0.1425 0.2452 0.266 0.1147 0.2316]
    [0.1207 0.1022 0.1805 0.1561 0.4404]
    [0.1027 0.0621 0.1211 0.1819 0.5324]
    [0.1879 0.2259 0.2668 0.1207 0.1986]
    [0.1269 0.1581 0.2004 0.1604 0.3541]
    [0.1083 0.1549 0.1723 0.1831 0.3814]
    [0.1032 0.1054 0.1893 0.178 0.424]
    [0.1974 0.1311 0.1691 0.1179 0.3845]
    [0.4267 0.1369 0.1376 0.1032 0.1957]
    [0.5025 0.1457 0.1364 0.0883 0.1272]
    [0.1557 0.1301 0.1813 0.1718 0.3611]
    [0.0992 0.2558 0.2506 0.1072 0.2873]
    [0.1306 0.1196 0.2143 0.1223 0.4132]
    [0.1159 0.2439 0.2652 0.125 0.25]
    [0.1833 0.0774 0.1222 0.1368 0.4804]
    [0.2269 0.043 0.0905 0.1196 0.52]
  ]
  open-output
  
  ;; create agents
  setup-agents
  ;; grow social network via Schelling-like reshuffling of agents in the (social) space
  setup-social-network 

  reset-ticks

  ;; set variables for output measures
  set num-turtles array:from-list n-values length ethnicity [0]
  foreach (n-values length ethnicity [?]) [
    array:set num-turtles ? count turtles with [item 2 traits = (item ? ethnicity)]
  ]
  set married-turtles array:from-list n-values length ethnicity [0]
  set out-percent array:from-list n-values length ethnicity [0]
  set cross-ethnic 0

end

to setup-agents
  repeat num-agents [
    create-new-agent pick-age-from-distribution
  ]
  ;; trace three random turtles
  foreach n-values 3 [random num-agents] [
    set traced-turtles lput ? traced-turtles
  ]
  ask turtles [
    trace (word "Hello, my traits are " traits) 
  ]
end

to create-new-agent [age]
  crt 1 [
    let my-gender random 2
    let my-age age
    let my-eth sample-empirical-distribution e-proportions [0 1 2 3]
    let my-comp precision random-normal 0.5 0.2 2 ;;precision random-float 1.0 2 ;;############### CHANGED to NORMAL DIST from RANDOM UNIFORM
    let my-educ pick-education-level-from-distribution my-eth my-gender
    set traits (list item my-gender gender my-age item my-eth ethnicity my-comp my-educ) ;;item my-educ education-levels)
    set shape item my-gender g-shape
    set size scale-size my-age
    set color item my-eth e-colour
    move-to free-patch-near-my-eth my-eth 
    set preferences setup-preferences-from (list my-gender my-age my-eth my-comp my-educ)
    set partner nobody
    set p-candidate nobody
    set already-tried? false
    set recently-dated [[][]]
    set date-counter 0
    set marriage-tick -1
    set dating-tick -1
  ]
end

to-report free-patch-near-my-eth [eth]
  ;; each ethnicity is assigned a centre: w in lower-left quadrant, x upper-left, y upper-right, z lower-right
  ;; the first of each ethnicity will take that patch
  if (not any? turtles with [item 2 traits = item eth ethnicity]) [
    report patch (item eth start-x-positions) (item eth start-y-positions)
    stop
  ]
  ;; other agents will find an empty patch next to a random turtle of their own ethnicity
  let mate one-of other turtles with [item 2 traits = item eth ethnicity and count neighbors with [not any? turtles-here] > 0]
  if (mate = nobody) [
    ;; found none, so just pick a random empty patch
    report one-of patches with [not any? turtles-here]
    stop
    ;;set mate one-of other turtles with [count neighbors with [not any? turtles-here] > 0]
  ]
  set xcor [xcor] of mate
  set ycor [ycor] of mate
  report one-of patches in-radius 2 with [not any? turtles-here]
end   

to-report setup-preferences-from [t-list] 
  let p-list []
  ;; 1. prefer opposite gender
  set p-list lput item (((item 0 t-list) + 1) mod 2) gender p-list
  ;; 2. prefer an age range around my own age; 
  ;; range between 2 and 10 years
  ;;let age-range 2 + random 9
  ;; range dictated by normal distribution with mean 1.3 and sd 6.34 
  let age-range abs (random-normal 1.3 6.34)
  set p-list lput (list ((item 1 t-list) - age-range) ((item 1 t-list) + age-range)) p-list
  ;; 3. prefer my own ethnicity
  ;; assign random preference values between 0 and 1 to all four ethnicities; making sure that the highest is with own eth
  let eth-pref-list generate-eth-prefs (item 2 t-list)
  set p-list lput eth-pref-list p-list
  ;; 4. prefer a range around my own compatibility score; range between 0.05 and 0.15
  let range 0.05 + random-float 0.1
  set p-list lput (list max (list (precision ((item 3 t-list) - range) 2) 0.0) min (list (precision ((item 3 t-list) + range) 2) 1.0)) p-list 
  ;; 5. prefer same or higher/lower education level according to normal distribution with mean 0 (same level) and standard deviation set via model parameter sd-education-pref
  set p-list lput random-normal 0 sd-education-pref p-list
  report p-list
end   

to setup-social-network
  ;; make links with (mostly) same ethnicity (according to initial position)
  ask turtles [
    make-new-links
  ]

  ;; run a Schelling-type segregation by age for a while
  repeat 50 [
    ask turtles [
      segregate-for 1
    ]
  ]
  ask turtles [
    make-new-links
  ]

  ask links [ set hidden? true ]
end

to go
  show ticks
  reset-variables
  ;; every five years: output social network of turtles
  if (file-output? and ticks mod (ticks-per-year * 5) = 0) [
    export-nodes
    export-edges
  ]
  
  ;; let unmarried/not dating agents find a potential partner
  let unmarried (turtles with [partner = nobody])
  let singles (unmarried with [not currently-dating?])
  ask singles [
    find-potential-partner
  ]
  ;; let all agents who found a potential partner go on a blind date with them
  ask singles with [p-candidate != nobody] [
    blind-date
  ]
  ;; let agents who are both satisfied with their blind date move on to the next stage
  ask singles with [satisfied? and [satisfied?] of p-candidate] [
    start-dating
  ]
  ;; let agents who have been dating for a while decide if they want to marry
  ask unmarried with [dating-tick + dating-duration = ticks] [
    try-for-marriage
  ]
  
  ;; let everyone who didn't date successfully update their preferences
  ask unmarried with [p-candidate = nobody or not satisfied?] [
    update-preferences-after-unsuccessful-date
  ]
  ;; give them a chance to grow their social network
  ask turtles [
    grow-network
  ]
  
  tick
  update-age
  update-output-variables
end

to reset-variables
  ask turtles with [partner = nobody and not currently-dating?] [
    set already-tried? false
    set p-candidate nobody
    set satisfied? false
    free-old-dates (ticks - second-chance-interval)
  ]
end

to update-age
  if (ticks mod ticks-per-year = 0) [
    ask turtles [
      let my-age (item 1 traits)
      set traits replace-item 1 traits (my-age + 1)
      set size scale-size my-age
    ]
  ]    
end

to update-output-variables
  foreach (n-values length ethnicity [?]) [
    array:set married-turtles ? count turtles with [item 2 traits = (item ? ethnicity) and partner != nobody]
    array:set out-percent ? cross-ethnic-marriage-proportion-for (item ? ethnicity) * 100
  ]
  set cross-ethnic ifelse-value (total-marriages > 0) [cross-ethnic-marriages / total-marriages * 100] [0]
end

;; ------------ dating related stuff -----------------------------------

to find-potential-partner
  if (p-candidate != nobody) [ stop ]  ;; only search for potential partners if nobody has already asked me
                                       ;; this makes sure that each agent only has up to ONE blind date per tick
  let my-list potential-partners-dynamic-nipple
  if (not empty? my-list) [
    ;; select one to ask out for a date -- for now: just pick a random one
    set p-candidate one-of my-list
    trace (word "found potential partner " p-candidate)
    block-a-date p-candidate
    ask p-candidate [
      set p-candidate myself
      trace (word "was found as potential partner by " myself)
      block-a-date myself
    ]
    ;; determine network distance between myself and p-candidate for model statistics
    update-dates-level
  ]
end

to-report potential-partners-dynamic-nipple
  let n 3
  ;; pick up to 3 potential partners, working your way out from direct links to extended network inside love radar
  let preferred-gender (item 0 preferences)
  let preferred-age-lower first (item 1 preferences)
  let preferred-age-upper last (item 1 preferences)
  let basic-set turtle-set self
  let i 1
  let plist no-turtles
  while [count plist < n and i <= love-radar] [    
    let pplist (turtle-set [link-neighbors] of basic-set) with [p-candidate = nobody and partner = nobody   
                                                                and not member? myself recently-dated 
                                                                and (item 0 traits) = preferred-gender 
                                                                and (item 1 traits) >= preferred-age-lower and (item 1 traits) <= preferred-age-upper]
    ;; if that leaves enough agents to choose from, add random selection (max: n - length plist; min: count pplist) to plist
    let num min (list (count pplist) (n - count plist)) 
    if (num > 0) [
      set plist (turtle-set plist (n-of num pplist))
    ]
    ;; prepare to extend search outwards
    set i i + 1 
    set basic-set turtle-set ([link-neighbors] of basic-set)
  ]
  report sort plist
end

to-report potential-partners-slim
  ;; do not collect ALL acquaintances inside love radar, instead pick a random person per level
  ;; this speeds up the execution of the model but should slow down the dating process nicely
  let pp-list no-turtles
  let chosen self
  let old nobody
  repeat love-radar [
    set old chosen
    set chosen one-of (([link-neighbors] of chosen) with [p-candidate = nobody and partner = nobody and not member? (turtle who) recently-dated])
    ifelse (chosen != nobody) [
      set pp-list (turtle-set pp-list chosen)
    ][
      set chosen one-of ([link-neighbors] of old)
    ]
  ]
  ;; filter list of potential partners: remove anyone I recently dated
  ;;set pp-list remove-recently-dated-from pp-list
  ;; do the rest of the filtering
  report filter-potential-partner-list sort pp-list
end
  
to-report potential-partners
  ;; depending on the love-radar (= distance to look in social network), gather all contacts
  ;; if love-radar = 1, only look at direct neighbours, if love-rader = 2, include direct friends of friends, and so on
  let pp-list gather-acquaintances love-radar
  ;; filter my social network for agents who fall within my preferences (gender/age/ethnicity)
  ;; i.e. they are of my preferred gender (either M or F), their age is within my preferred age range
  ;; and their ethnicity is ???
  ;; preparations: 
  ;; (a) only look at friends who aren't already "taken"
  set pp-list sort ((turtle-set pp-list) with [p-candidate = nobody and partner = nobody])
  ;;set pp-list sort (link-neighbors with [p-candidate = nobody and partner = nobody]) ;; only look at friends who aren't already "taken"
  ;; (b): remove everyone I recently dated
  set pp-list remove-recently-dated-from pp-list
  ;; do the filtering
  report filter-potential-partner-list pp-list
end 

to-report filter-potential-partner-list [pp-list]
  ;; 1. step: filter for gender
  set pp-list filter [item 0 [traits] of ? = item 0 preferences] pp-list
  if (empty? pp-list) [
    trace (word "no potential partners in my social network")
    report []
  ]
  ;; 2. step: filter for age range
  let lower-bound first (item 1 preferences)
  let upper-bound last (item 1 preferences)
  let temp filter [item 1 [traits] of ? >= lower-bound and item 1 [traits] of ? <= upper-bound] pp-list
  if (empty? temp) [
    report pp-list   ;; report just filter for gender
  ]
  set pp-list temp
  ;; 3. step: filter for ethnicity
  ;; QUESTION: HOW??
  report pp-list
end  

to blind-date
  set satisfied? satisfied-with? p-candidate
  trace (word "blind-date with " p-candidate " is satisfactory? " satisfied?)
  set date-counter date-counter + 1
end

to-report satisfied-with? [other-agent]
  ;; an agent is satisfied with another agent if that agent is 
  ;; (a) of the preferred gender and
  ;; (b) inside the preferred age range and
  ;; (c) of an "acceptable" ethnicity. PROBLEM: How do we derive "acceptable" from the preference value?
  ;; --> One solution: interpret preference value as probability
  ;; (d) and has the preferred education level
  let answer [first traits] of other-agent = first preferences
  set answer answer and is-in-range? ([item 1 traits] of other-agent) (item 1 preferences) 
  let eth [item 2 traits] of other-agent
  let index position eth ethnicity
  set answer answer and (random-float 1.0 < item index (item 2 preferences))
  ;; the preference tells me about the maximum acceptable difference between my education level and the other-agent's education level
  let my-pref (item 4 preferences)
  ;; let's first compute the difference between your and mine education levels
  let diff [item 4 traits] of other-agent - (item 4 traits)
  ;;set answer answer and ((my-pref >= 0 and diff >= 0 and diff <= my-pref) or (my-pref < 0 and diff <= 0 and diff >= my-pref))
  set answer answer and (abs diff <= abs my-pref)
  report answer
end

to start-dating
  trace (word "start dating " p-candidate)
  set dating-tick ticks
  ;; decide length of dating according to normal distribution
  set dating-duration years-to-ticks (random-normal mean-dating sd-dating)
  ;; blind date was successful --> boost ethnicity preference for potential partner's ethnicity
  boost-ethnicity-pref p-candidate  
end
  
to try-for-marriage
  if (already-tried?) [ stop ]
  ;; blind date was successful --> boost ethnicity preference for potential partner's ethnicity
;;  boost-ethnicity-pref p-candidate
;;  ask p-candidate [
;;    boost-ethnicity-pref myself
;;  ]
  ;; check if compatibility scores are within the preferred range
  ifelse (compatible? p-candidate self) [
    trace (word p-candidate " and I are compatible")
    ask p-candidate [
      trace (word p-candidate " and I are compatible")
    ]
    ;; let's see if we are meant for each other
    let marriage-probability 1.0 - (abs (item 3 traits) - [item 3 traits] of p-candidate)  ;; the closer the compatibility scores, the closer the probability is to 1.0 
    ifelse (random-float 1.0 < marriage-probability) [
      ;; SUCCESS
      trace (word "AND WE WILL MARRY, HURRAH!")
      set partner p-candidate
      update-dates-before-marriage date-counter
      update-marriage-level
      set marriage-tick ticks
      ask p-candidate [
        trace (word "AND WE WILL MARRY, HURRAH!")
        set partner myself
        update-dates-before-marriage date-counter
        set marriage-tick ticks
      ]
      ;; link partners directly (if not yet linked)
      create-link-with p-candidate
      ;; make link between them visible to show they are married
      ask link [who] of self [who] of p-candidate [
        set hidden? false
        set thickness 0.2
      ]
      ;; update marriage counts
      update-marriage-counts (position (item 2 traits) ethnicity) (position [item 2 traits] of p-candidate ethnicity)
    ] [
      ;; nope, it wasn't meant to be
      trace (word "but we're not meant to marry, so we're breaking up")
      ;; --> update compatibility range
      update-compatibility-range 0.05
      ask p-candidate [
        trace (word "but we're not meant to marry, so we're breaking up")
        update-compatibility-range 0.05
      ]
      ;; stop dating
      stop-dating
    ]
  ] [
    trace (word p-candidate " and I are not compatible, so we're breaking up")
    ;; --> update compatibility range
    update-compatibility-range 0.05
    ask p-candidate [
      trace (word p-candidate " and I are not compatible, so we're breaking up")
      update-compatibility-range 0.05
    ]
    ;; stop dating
    stop-dating
  ]
  set already-tried? true
  ask p-candidate [
    set already-tried? true
  ]
end

to-report compatible? [agent1 agent2]
  ;; the two agents are compatible if a1's compatibility score falls within a2's preferred range and vice versa
  report (is-in-range? [item 3 traits] of agent1 [item 3 preferences] of agent2) and
         (is-in-range? [item 3 traits] of agent2 [item 3 preferences] of agent1)
end

to boost-ethnicity-pref [other-agent]
  if (date-counter mod update-threshold != 0) [ stop ] ;; only update after so many dates
  let eth [item 2 traits] of other-agent
  trace (word "boosting my ethnicity preference for " eth " from " (item 2 preferences))
  update-ethnicity-pref eth 0.01
  trace (word " -- to now " (item 2 preferences))
end

to dampen-ethnicity-pref [other-agent]
  if (date-counter mod update-threshold != 0) [ stop ] ;; only update after so many dates
  let eth [item 2 traits] of other-agent
  trace (word "dampening my ethnicity preference for " eth " from " (item 2 preferences))
  update-ethnicity-pref eth -0.01
  trace (word " -- to now " (item 2 preferences))
end

to update-ethnicity-pref [eth value]
  let index position eth ethnicity
  let temp (item 2 preferences)
  set temp replace-item index temp max (list 0.0 min (list 1.0 precision ((item index temp) + value) 2))
  set preferences replace-item 2 preferences temp
end

to update-compatibility-range [value]
  if (date-counter mod update-threshold != 0) [ stop ] ;; only update after so many dates  
  trace (word "updating compatibilty range from " (item 3 preferences))
  update-preferences-range 3 value 0.0 1.0
  trace (word "-- to now " (item 3 preferences))
end

to update-age-range [value]
  if (date-counter mod update-threshold != 0) [ stop ] ;; only update after so many dates
  trace (word "updating age range preference from " (item 1 preferences))
  update-preferences-range 1 value 0 100
  trace (word "-- to now " (item 1 preferences))
end

to update-preferences-range [index value r-min r-max]
  let temp (item index preferences)
  set temp replace-item 0 temp max (list r-min precision ((first temp) - value) 2)
  set temp replace-item 1 temp min (list r-max precision ((last temp) + value) 2)
  set preferences replace-item index preferences temp
end

to update-preferences-after-unsuccessful-date
  if (date-counter mod update-threshold != 0) [ stop ] ;; only update after so many dates
  trace (word "updating preferences after unsuccessful date")
  ;; widen age range
  update-age-range 0.1
  ;; negative influence on ethnicity of potential partner
  if (p-candidate != nobody) [
    dampen-ethnicity-pref p-candidate
  ]
end
      
to block-a-date [other-agent]
  ;; add other-agent to list of agents
  set recently-dated replace-item 0 recently-dated (lput other-agent (first recently-dated)) 
  ;; add current tick to list of ticks
  set recently-dated replace-item 1 recently-dated (lput ticks (last recently-dated))
end

to free-old-dates [cut-off-time]
  if (cut-off-time < 0) [ stop ]
  let times last recently-dated
  let agents first recently-dated
  let i 0
  while [(i < length times) and (item i times < cut-off-time)] [
    set i i + 1
  ]
  repeat i [
    set times remove-item 0 times
    set agents remove-item 0 agents
  ]
  set recently-dated (list agents times)
end

to-report remove-recently-dated-from [plist]
  if (empty? recently-dated) [
    report plist
  ]
  let already-dated (first recently-dated)
  let cleared-list filter [not member? ? already-dated] plist
  report cleared-list
end

to stop-dating
  set dating-tick -1
  set dating-duration 0
  ask p-candidate [
    set dating-tick -1
    set dating-duration 0
  ]
end
  
;; ------------- social network stuff ----------------------------------

to make-new-links
  ;; make links with some of my neighbours
  trace (word "is " (item 0 traits) " and got " count turtles-on neighbors " neighbours, " count (turtles-on neighbors) with [(item 0 traits) = "F"] " female and " count (turtles-on neighbors) with [(item 0 traits) = "M"] " male")
  ;; slight preference for same gender
  let same-gender-pref 0.1
  foreach sort (turtles-on neighbors) [
    let same-gender ifelse-value ([item 0 traits] of ? = [item 0 traits] of self) [1] [0]
    if (random-float 1.0 < (0.4 + same-gender * same-gender-pref)) [
      create-link-with ? [
        set hidden? true
      ]
    ]
  ]
    
;  create-links-with turtles-on neighbors [
;    set hidden? true
;  ]
end

to grow-network
  ;; with a small probability, drop a random link
  if (random-float 1.0 < 0.05 and count my-links > 1) [ ;; make sure there are actually any links to be dropped
    ask one-of my-links with [hidden? = true] [die]  ;; make sure we don't drop the marriage link
    trace (word "dropping a link")
    set dropped-links dropped-links + 1
  ]
  ;; with a medium probability, add a new link
  if (random-float 1.0 < new-link-chance) [
    ifelse (random-float 1.0 < 0.95) [
      ;; either make friends with the most similar friend of a friend
      befriend-similar
    ][
      ;; or make friends with a random stranger of similar age
      befriend-stranger
    ]
  ]
end

to befriend-similar  
  ;; make a link with the most similar friend of a friend
  ;; pick a random friend
  let friend one-of link-neighbors
  ;; get their friends (without myself)
  let p-friends sort other [link-neighbors] of friend
  ;; if there aren't any friends, stop
  if (empty? p-friends) [stop]
  ;; score friends according to similarity to myself (with regard to age and ethnicity), then pick the most similar
  let similarity-scores compute-similarity p-friends
  let best position (min similarity-scores) similarity-scores
  set friend item best p-friends
  create-link-with friend [
    set hidden? true
  ]
  trace (word "creating new link with " friend " who has a similarity-score of " (item best similarity-scores))
end

to befriend-stranger
  ;; make a link with a stranger of similar age (+- 5 years)
  let range (list ((item 1 traits) - 5) ((item 1 traits) + 5))  ;;(item 1 preferences)
  create-link-with one-of other turtles with [is-in-range? (item 1 traits) range] [
    set hidden? true
  ]
  trace (word "creating new link with a stranger")
end

to-report compute-similarity [alist]
  let scores []
  foreach alist [
    set scores lput (similarity-score ?) scores
  ]
  report scores  
end

to-report similarity-score [other-agent]
  ;; how similar is other-agent to me with regard to age and ethnicity and education?
  ;; 1. compute absolute age distance and normalise to [0,1] (assuming max distance = (36-18) = 18)
  let dist abs ((item 1 traits) - [item 1 traits] of other-agent)
  set dist dist / 18
  ;; 2. compute ethnicity "distance" using preferences: 1 - preference for other-agent's ethnicity
  let e-dist 1 - (item (position [item 2 traits] of other-agent ethnicity) (item 2 preferences))
  ;; 3. compute absolute education distance and normalise to [0,1] (max distance is 4)
  let edu-dist abs ((item 4 traits) - [item 4 traits] of other-agent) / 4
  report precision ((dist + e-dist + edu-dist) / 3) 3
end

to-report gather-acquaintances [depth]
  if (depth <= 0) [
    report []
  ]
  let aset link-neighbors ;; this is depth 1
  repeat (depth - 1) [
    set aset gather-and-merge-acquaintances-of (sort aset) 
  ]
  report sort aset  ;; NOTE: aset contains the agent itself, but that doesn't matter as long as we're in a strictly heterosexual world
end

to-report gather-and-merge-acquaintances-of [agent-list]
  let aset no-turtles
  foreach agent-list [
    set aset (turtle-set aset [link-neighbors] of ?)
  ]
  report sort aset
end

to-report same-ethnic-links
  report count link-neighbors with [item 2 traits = [item 2 traits] of myself]
end

to-report same-ethnic-links-proportion
  let n count link-neighbors
  report ifelse-value (n > 0) [same-ethnic-links / n] [0]
end

;; ------------- Schelling-type model stuff -----------------------------

to segregate-for [trait]   ;; either age or ethnicity
  if (not happy-enough-with-current-position? trait Schelling-threshold) [
    move-to one-of patches with [not any? turtles-here]
  ]
end

to-report happy-enough-with-current-position? [trait threshold]
  ;; check how many of my Moore neighbours are similar to me 
  let my-neighbours turtle-set turtles-on neighbors
  if (count my-neighbours = 0) [ 
    report false
  ]
  ;; if trait = age, look if they fall within my preferred age-range
  ;; if trait = ethnicity, look if they have the same ethnicity
  let prop-similar 0
  if (trait = 2) [ ;; ethnicity
    let my-eth item trait traits
    set prop-similar count my-neighbours with [(item trait traits) = my-eth]
  ]
  if (trait = 1) [ ;; age
    let lower-bound first (item 1 preferences)
    let upper-bound last (item 1 preferences)
    set prop-similar count my-neighbours with [(item trait traits) >= lower-bound and (item trait traits) <= upper-bound]
  ]
  set prop-similar prop-similar / count my-neighbours
  if (prop-similar >= threshold) [
    report true
  ]
  report false
end

    
;;--------------- utils --------------------------------------------------

to update-dates-level
  ;; turtle procedure: find network distance from self to p-candidate
  let level -1
  let basic-set turtle-set self
  let found false
  while [level < love-radar and not found] [
    set basic-set turtle-set [link-neighbors] of basic-set
    if (member? p-candidate basic-set) [
      set found true
    ]
    set level level + 1
  ]
  set dates-level replace-item level dates-level (item level dates-level + 1)
  ;; update turtle attribute, used for determining marriage level
  set date-level level
  ask p-candidate [
    set date-level level
  ]
end  

to update-marriage-level
  set marriages-level replace-item date-level marriages-level (item date-level marriages-level + 1)
end

to open-output 
  if (not file-output?) [ stop ]
  let current-dir pathdir:get-current
  set sep pathdir:get-separator
  pathdir:create (word current-dir sep "experiments")
  file-open (word current-dir sep "experiments" sep "_hack.txt")
  file-write "."
  file-close-all
  let system-time pathdir:get-date-ms (word current-dir sep "experiments" sep "_hack.txt")
  file-delete (word current-dir sep "experiments" sep "_hack.txt")
  set output-dir (word current-dir sep "experiments" sep "run-" system-time)
  pathdir:create output-dir
end

to export-nodes
  if (not file-output?) [ stop ]
  file-open (word output-dir sep "nodes-" (ticks / ticks-per-year) ".csv")
  file-print "id,sex,age,ethnicity,compatibility,preferences"
  ask turtles [
    file-print (word who "," (item 0 traits) "," (item 1 traits) "," (item 2 traits) "," (item 3 traits) "," preferences)
  ]
  file-close
end

to export-edges
  if (not file-output?) [ stop ]
  file-open (word output-dir sep "edges-" (ticks / ticks-per-year) ".csv")
  file-print "source,target,type,partner"
  ask links [
    file-print (word ([who] of end1) "," ([who] of end2) ",undirected," hidden?)
  ]
  file-close
end


to-report sample-empirical-distribution [probabilities values]
  ;; probabilities are not accumulated but add up to 1.0
  ;; there is a probability for each value
  let k random-float 1.0
  let i 0
  let lower-bound 0
  while [i < length probabilities] [
    ifelse (k < precision (lower-bound + item i probabilities) 4) [
      report item i values
    ] [
      set lower-bound precision (lower-bound + item i probabilities) 4
      set i i + 1
    ]
  ]
  show (word "ERROR in sample-empirical-distribution with probs " probabilities " and values " values " // k = " k " and i = " i)
  report -1
end

to-report generate-eth-prefs [own-eth]
  ;; generate a random number for all ethnicity
  let plist n-values length ethnicity [precision (random-float 1.0) 2]
  ;; determine index of max value
  let max-index position (max plist) plist
  if (max-index != own-eth) [
    ;; swap values so that value for own ethnicity is max value
    let temp (item max-index plist)
    set plist replace-item max-index plist (item own-eth plist)
    set plist replace-item own-eth plist temp
  ]
  report plist
end

to-report pick-education-level-from-distribution [eth sex]
  ;; eth is one of 0 = w, 1 = x, 2 = y, 3 = z
  ;; sex is one of 0 = male, 1 = female
  ;; dependent on eth pick the correct associated ethnic group, specified via the choosers in the interface
  let eth-group ifelse-value (eth = 0) [ethnicity-w] [ifelse-value (eth = 1) [ethnicity-x] [ifelse-value (eth = 2) [ethnicity-y] [ethnicity-z]]]
  ;; pick the correct education proportion entry
  let entry position eth-group ethnic-groups
  let education-list ifelse-value (sex = 0) [education-proportions-men] [education-proportions-women]
  ;; turn it into a distribution and sample it to get a level
  let level sample-empirical-distribution (item entry education-list) n-values (length education-levels) [?]
  report level
end
  
to-report pick-age-from-distribution
  let i sample-empirical-distribution age-distribution-probs n-values (length age-distribution-probs) [?]
  ;; i is the index for the upper bound
  let upper-bound item i age-distribution-bounds
  ;; lower bound is 18 (if i = 0) or item (i - 1) age-distribution-bounds (if i > 0)
  let lower-bound ifelse-value (i = 0) [18] [item (i - 1) age-distribution-bounds]
  
  report (lower-bound + random (upper-bound - lower-bound))
end

to-report is-in-range? [value range]
  report value >= (first range) and value <= (last range)
end

to-report total-marriages
  let total 0
  foreach n-values (length ethnicity) [?] [
    let i ?
    foreach n-values (length ethnicity) [?] [
      let j ?
      set total total + array:item (array:item marriage-counts i) j
    ]
  ]
  report total
end

to-report same-ethnic-marriages
  let total 0
  foreach n-values (length ethnicity) [?] [
    set total total + array:item (array:item marriage-counts ?) ?
  ]
  report total
end

to-report cross-ethnic-marriages
  report total-marriages - same-ethnic-marriages
end

to-report cross-ethnic-marriage-proportion-for [eth]
  ;; compute w_married - w-w / w_married = w-x + w-y + w-z / w_married (if eth = w)
  let i (position eth ethnicity)
  let same-m count-marriages i i
  let total-m count turtles with [item 2 traits = eth and partner != nobody] / 2
  if (total-m > 0) [
    report (total-m - same-m) / total-m
  ]
  report 0
end
  
to-report same-ethnic-marriage-proportion-for [eth]
  ;; compute w-w / w_married (if eth = w)
  let i (position eth ethnicity)
  let same-m count-marriages i i
  let total-m count turtles with [item 2 traits = eth and partner != nobody] / 2
  if (total-m > 0) [
    report same-m / total-m
  ]
  report 0
end
  
to-report marriage-proportion [eth1 eth2]
  let i (position eth1 ethnicity)
  let j (position eth2 ethnicity)
  let m count-marriages i j
  let n total-marriages
  if (n = 0) [
    report 0
  ]
  report m / n
end

to-report count-marriages [i j]
  let m array:item (array:item marriage-counts i) j
  if (i != j) [
    set m m + array:item (array:item marriage-counts j) i
  ]
  report m
end

to update-marriage-counts [i j]
  let old-value array:item (array:item marriage-counts i) j
  array:set (array:item marriage-counts i) j (old-value + 1)
end

to-report init-marriage-counts
  let basic-list n-values length ethnicity [0]
  let complex-list n-values length ethnicity [array:from-list basic-list]
  report array:from-list complex-list
end

to update-dates-before-marriage [num-dates]
  let dates (first dates-before-marriage) + num-dates
  let married (last dates-before-marriage) + 1
  set dates-before-marriage (list dates married)
end
  
to-report avg-dates-before-marriage
  if (last dates-before-marriage = 0) [
    report 0
  ]
  report (first dates-before-marriage) / (last dates-before-marriage)
end

to-report age-at-marriage
  if (marriage-tick < 0) [
    report -1
  ]
  ;; compute age at marriage from current age, ticks since marriage-tick and ticks-per-year
  let current-age-in-ticks (item 1 traits) * ticks-per-year + (ticks mod ticks-per-year)
  report round ((current-age-in-ticks - (ticks - marriage-tick)) / 12)
end

to trace [output-text]
  if (member? who traced-turtles) [
    show output-text
  ]
end

to-report scale-size [value]
  report value / 30
end

to-report predicted-cross-rate
  ;; compute probability of same-ethnic marriages
  let same 0
  foreach n-values (length e-proportions) [?] [
    set same same + (item ? e-proportions) * (item ? e-proportions)
  ]
  report 100 - (same * 100)
end

to-report compatibility-width
  let range item 3 preferences
  report (last range - first range)
end

to-report own-eth-pref
  report (item (position (item 2 traits) ethnicity) (item 2 preferences))
end

to-report prop-same-gender-links
  let my-gender (item 0 traits)
  let num-same length (filter [[item 0 traits] of ? = my-gender] [other-end] of my-links)
  report num-same / count my-links
end
   
to-report prop-single [eth]
  let n count turtles with [item 2 traits = eth]
  if (n = 0) [
    report 0
  ]
  report count turtles with [partner = nobody and (item 2 traits) = eth] / n * 100
end  
  
to-report currently-dating?
  report dating-tick >= 0
end    

to-report years-to-ticks [years]
  ;; convert years to ticks using ticks-per-year
  report round (years * ticks-per-year)
end
@#$#@#$#@
GRAPHICS-WINDOW
210
10
715
536
16
16
15.0
1
10
1
1
1
0
1
1
1
-16
16
-16
16
0
0
1
ticks
30.0

SLIDER
13
13
185
46
num-agents
num-agents
100
1000
1000
10
1
NIL
HORIZONTAL

BUTTON
10
120
76
153
setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

INPUTBOX
14
54
188
114
eth-proportions
[0.498 0.179 0.13 0.193]
1
0
String

PLOT
735
11
935
161
Link Distribution
num links
NIL
0.0
50.0
0.0
200.0
true
false
"set-histogram-num-bars 20" ""
PENS
"default" 1.0 1 -16777216 true "" "histogram [count link-neighbors] of turtles"

BUTTON
140
120
203
153
go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

MONITOR
738
172
813
217
marriages
count turtles with [partner != nobody] / 2
17
1
11

MONITOR
824
172
935
217
cross-ethnic %
cross-ethnic-marriages / total-marriages * 100
2
1
11

BUTTON
75
120
138
153
step
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

INPUTBOX
15
575
170
635
ticks-per-year
12
1
0
Number

INPUTBOX
16
642
171
702
second-chance-interval
6
1
0
Number

SLIDER
15
410
171
443
love-radar
love-radar
1
5
2
1
1
NIL
HORIZONTAL

SLIDER
15
448
172
481
update-threshold
update-threshold
1
10
5
1
1
NIL
HORIZONTAL

MONITOR
740
228
901
273
# dates before marriage
avg-dates-before-marriage
1
1
11

PLOT
740
284
940
434
Age at Marriage
age
count
18.0
45.0
0.0
50.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" "histogram [age-at-marriage] of turtles with [partner != nobody]"

PLOT
741
442
941
592
Age of Singles
age
count
18.0
50.0
0.0
50.0
true
false
"" ""
PENS
"w" 1.0 1 -955883 true "" "histogram [item 1 traits] of turtles with [partner = nobody and (item 2 traits) = \"w\"]"
"x" 1.0 1 -13840069 true "" "histogram [item 1 traits] of turtles with [partner = nobody and (item 2 traits) = \"x\"]"
"y" 1.0 1 -1184463 true "" "histogram [item 1 traits] of turtles with [partner = nobody and (item 2 traits) = \"y\"]"
"z" 1.0 1 -11221820 true "" "histogram [item 1 traits] of turtles with [partner = nobody and (item 2 traits) = \"z\"]"

MONITOR
944
172
1078
217
max cross-ethnic %
predicted-cross-rate
2
1
11

MONITOR
1065
230
1146
275
NIL
count links
0
1
11

PLOT
951
285
1151
435
% married
ticks
%
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"w" 1.0 0 -955883 true "" "plot 100 - (prop-single \"w\")"
"x" 1.0 0 -13840069 true "" "plot 100 - (prop-single \"x\")"
"y" 1.0 0 -1184463 true "" "plot 100 - (prop-single \"y\")"
"z" 1.0 0 -11221820 true "" "plot 100 - (prop-single \"z\")"

PLOT
950
443
1150
593
% single
ticks
%
0.0
10.0
0.0
100.0
true
false
"" ""
PENS
"w" 1.0 0 -955883 true "" "plot prop-single \"w\""
"x" 1.0 0 -10899396 true "" "plot prop-single \"x\""
"y" 1.0 0 -1184463 true "" "plot prop-single \"y\""
"z" 1.0 0 -11221820 true "" "plot prop-single \"z\""

SLIDER
13
492
185
525
mean-dating
mean-dating
0
5
1
0.1
1
years
HORIZONTAL

SLIDER
13
531
185
564
sd-dating
sd-dating
0
2
1
0.1
1
years
HORIZONTAL

TEXTBOX
216
546
366
564
Overall marriage %
11
0.0
1

MONITOR
220
565
277
610
w-w
marriage-proportion \"w\" \"w\" * 100
2
1
11

MONITOR
285
565
342
610
w-x
marriage-proportion \"w\" \"x\" * 100
2
1
11

MONITOR
350
565
407
610
w-y
marriage-proportion \"w\" \"y\" * 100
2
1
11

MONITOR
415
565
472
610
w-z
marriage-proportion \"w\" \"z\" * 100
2
1
11

MONITOR
285
615
342
660
x-x
marriage-proportion \"x\" \"x\" * 100
2
1
11

MONITOR
350
615
407
660
x-y
marriage-proportion \"x\" \"y\" * 100
2
1
11

MONITOR
415
615
472
660
x-z
marriage-proportion \"x\" \"z\" * 100
2
1
11

MONITOR
350
665
407
710
y-y
marriage-proportion \"y\" \"y\" * 100
2
1
11

MONITOR
415
665
472
710
y-z
marriage-proportion \"y\" \"z\" * 100
2
1
11

MONITOR
415
715
472
760
z-z
marriage-proportion \"z\" \"z\" * 100
2
1
11

MONITOR
600
565
657
610
w-out
cross-ethnic-marriage-proportion-for \"w\" * 100
2
1
11

TEXTBOX
530
545
680
563
Group marriage %
11
0.0
1

MONITOR
530
565
587
610
w-in
same-ethnic-marriage-proportion-for \"w\" * 100
2
1
11

MONITOR
530
615
587
660
x-in
same-ethnic-marriage-proportion-for \"x\" * 100
2
1
11

MONITOR
600
615
657
660
x-out
cross-ethnic-marriage-proportion-for \"x\" * 100
2
1
11

MONITOR
530
665
587
710
y-in
same-ethnic-marriage-proportion-for \"y\" * 100
2
1
11

MONITOR
530
715
587
760
z-in
same-ethnic-marriage-proportion-for \"z\" * 100
2
1
11

MONITOR
600
665
657
710
y-out
cross-ethnic-marriage-proportion-for \"y\" * 100
2
1
11

MONITOR
600
715
657
760
z-out
cross-ethnic-marriage-proportion-for \"z\" * 100
2
1
11

CHOOSER
15
160
192
205
ethnicity-w
ethnicity-w
"White British" "White Irish" "Other White" "White and Black Carib" "White and Black Afric" "White and Asian" "Other Mixed" "Indian" "Pakistani" "Bangladeshi" "Other Asian" "Black Caribbean" "Black African" "Other Black" "Chinese" "Other Ethnic Group"
0

CHOOSER
15
260
192
305
ethnicity-y
ethnicity-y
"White British" "White Irish" "Other White" "White and Black Carib" "White and Black Afric" "White and Asian" "Other Mixed" "Indian" "Pakistani" "Bangladeshi" "Other Asian" "Black Caribbean" "Black African" "Other Black" "Chinese" "Other Ethnic Group"
9

CHOOSER
15
210
192
255
ethnicity-x
ethnicity-x
"White British" "White Irish" "Other White" "White and Black Carib" "White and Black Afric" "White and Asian" "Other Mixed" "Indian" "Pakistani" "Bangladeshi" "Other Asian" "Black Caribbean" "Black African" "Other Black" "Chinese" "Other Ethnic Group"
7

CHOOSER
15
310
192
355
ethnicity-z
ethnicity-z
"White British" "White Irish" "Other White" "White and Black Carib" "White and Black Afric" "White and Asian" "Other Mixed" "Indian" "Pakistani" "Bangladeshi" "Other Asian" "Black Caribbean" "Black African" "Other Black" "Chinese" "Other Ethnic Group"
15

PLOT
745
605
945
755
Number of links
ticks
num links
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" "plot count links"
"pen-1" 1.0 0 -7500403 true "" "plot dropped-links"

MONITOR
965
610
1032
655
Distance 1
item 0 dates-level
17
1
11

MONITOR
965
660
1032
705
Distance 2
item 1 dates-level
17
1
11

MONITOR
965
710
1032
755
Distance 3
item 2 dates-level
17
1
11

TEXTBOX
965
595
1005
613
Dates
11
0.0
1

MONITOR
1050
610
1117
655
Distance 1
item 0 marriages-level
17
1
11

MONITOR
1050
660
1117
705
Distance 2
item 1 marriages-level
17
1
11

MONITOR
1050
710
1117
755
Distance 3
item 2 marriages-level
17
1
11

TEXTBOX
1050
595
1105
613
Marriages
11
0.0
1

PLOT
945
10
1145
160
Proportion of intra-ethnic links
NIL
NIL
0.0
10.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 0 -955883 true "" "plot mean [same-ethnic-links-proportion] of turtles with [item 2 traits = \"w\"]"
"pen-1" 1.0 0 -13840069 true "" "plot mean [same-ethnic-links-proportion] of turtles with [item 2 traits = \"x\"]"
"pen-2" 1.0 0 -1184463 true "" "plot ifelse-value (any? turtles with [item 2 traits = \"y\"]) [mean [same-ethnic-links-proportion] of turtles with [item 2 traits = \"y\"]] [0]"
"pen-3" 1.0 0 -11221820 true "" "plot ifelse-value (any? turtles with [item 2 traits = \"z\"]) [mean [same-ethnic-links-proportion] of turtles with [item 2 traits = \"z\"]] [0]"

SLIDER
15
365
187
398
sd-education-pref
sd-education-pref
0
4
1
1
1
NIL
HORIZONTAL

SLIDER
15
705
187
738
new-link-chance
new-link-chance
0
1
0.5
0.05
1
NIL
HORIZONTAL

@#$#@#$#@
## WHAT IS IT?

This is version 0.6 of the **SCID Marriage Model**, aka **DITCH** (Diversity & Interethnic marriage - Trust, Culture & Homophily).

## HOW IT WORKS

In this model version, agents have five _traits_: gender (M/F), age, ethnicity (w,x,y,z), an abstract compatibility score (real number between 0-1), and an education level (integer number between 0 and 4). Their ethnicity is visualised as their colour (w: orange, x: green, y: yellow, z: cyan), their gender as their shape, and their age as their size (the older, the bigger).

Agents also have _preferences_ for partners with certain traits, expressed as (opposite) gender, an age range centred on their own age, preference values for each ethnicity (with the highest value for their own ethnicity), a compatibility range centred on their own compatibility score, and a preference of the difference in education level.

Each tick, agents try to find a potential partner and go on a blind date with them. If they are both satisfied with their date (with regard to gender/age/ethnicity/education) they keep dating each other for a while before they move on to the next stage. Here they check if they are compatible (i.e. if their compatibility scores fall within the other's preferred range) and if so, have a high chance of marrying each other.

## HOW TO USE IT

You can specify the number of agents via the _num-agents_ parameter and the proportion of the four ethnicities via the _eth-proportions_ parameter. Make sure that all proportions add up to 1.0. By assigning a particular ethnicity to each of the four abstract ones used in the model (_ethnicity-w_, _ethnicity-x_, _ethnicity-y_, _ethnicity-z_), agents are initialsed with an education level corresponding to a distribution pertaining to their ethnicity (obtained from the census 2001).

The agents' preference for difference in education level follows a normal distribution with mean 0 (= partner should have the same education level) and standard deviation set via the parameter _sd-education-pref_. Another normal distribution is used to determine the dating period. You can specify its mean (_mean-dating_) and standard deviation (_sd-dating_).

The _love-radar_ determines an agent's search range within its social network. If set to 1, the agent will only consider direct friends to find a potential partner. Set it to 2 or higher to allow agents access to friends of friends (of friends...).

Agents learn from their experiences in that unsuccessful dates make them update their preferences slightly. Via the parameter _update-threshold_ you can fine tune the agents' susceptibility to negative experiences: It will update its preferences after the specified number of unsuccessful dates.

You can also specify the number of ticks per year (_ticks-per-year_) and the number of ticks that have to elapse before an agent can date the same agent again (_second-chance-interval_). The _new-link-chance_ influences the speed with which the social networks grow. 

## THINGS TO NOTICE

(suggested things for the user to notice while running the model)

## THINGS TO TRY

(suggested things for the user to try to do (move sliders, switches, etc.) with the model)

## EXTENDING THE MODEL

(suggested things to add or change in the Code tab to make the model more complicated, detailed, accurate, etc.)

## NETLOGO FEATURES

(interesting or unusual features of NetLogo that the model uses, particularly in the Code tab; or where workarounds were needed for missing features)

## RELATED MODELS

(models in the NetLogo Models Library and elsewhere which are of related interest)

## CREDITS AND REFERENCES

This model was developed within the ESPRC-funded Social Complexity of Immigration and Diversity (SCID) project: http://scid-project.org/
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

female
false
0
Circle -7500403 true true 116 11 67
Polygon -7500403 true true 135 90 120 90 60 195 240 195 180 90 135 90
Rectangle -7500403 true true 105 195 135 285
Rectangle -7500403 true true 165 195 195 285

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

male
false
0
Circle -7500403 true true 116 11 67
Rectangle -7500403 true true 105 90 195 195
Rectangle -7500403 true true 105 195 135 285
Rectangle -7500403 true true 165 195 195 285

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.3
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="experiment" repetitions="10" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="120"/>
    <metric>item 0 array:to-list num-turtles</metric>
    <metric>item 1 array:to-list num-turtles</metric>
    <metric>item 2 array:to-list num-turtles</metric>
    <metric>item 3 array:to-list num-turtles</metric>
    <metric>item 0 array:to-list married-turtles</metric>
    <metric>item 1 array:to-list married-turtles</metric>
    <metric>item 2 array:to-list married-turtles</metric>
    <metric>item 3 array:to-list married-turtles</metric>
    <metric>item 0 array:to-list out-percent</metric>
    <metric>item 1 array:to-list out-percent</metric>
    <metric>item 2 array:to-list out-percent</metric>
    <metric>item 3 array:to-list out-percent</metric>
    <metric>cross-ethnic</metric>
    <enumeratedValueSet variable="eth-proportions">
      <value value="&quot;[0.498 0.179 0.13 0.193]&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ethnicity-w">
      <value value="&quot;White British&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ethnicity-x">
      <value value="&quot;Indian&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ethnicity-y">
      <value value="&quot;Bangladeshi&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ethnicity-z">
      <value value="&quot;Black African&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="love-radar">
      <value value="1"/>
      <value value="2"/>
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="update-threshold">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="second-chance-interval">
      <value value="6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="sd-dating">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ticks-per-year">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num-agents">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="sd-education-pref">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mean-dating">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="new-link-chance">
      <value value="0.5"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="network-growth" repetitions="1" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <final>file-close-all</final>
    <timeLimit steps="120"/>
    <metric>total-marriages</metric>
    <metric>same-ethnic-marriages</metric>
    <metric>cross-ethnic-marriages</metric>
    <metric>cross-ethnic-marriage-proportion-for "w"</metric>
    <metric>cross-ethnic-marriage-proportion-for "x"</metric>
    <metric>cross-ethnic-marriage-proportion-for "y"</metric>
    <metric>cross-ethnic-marriage-proportion-for "z"</metric>
    <metric>count links</metric>
    <metric>sum [same-ethnic-links] of turtles with [item 2 traits = "w"]</metric>
    <metric>sum [same-ethnic-links] of turtles with [item 2 traits = "x"]</metric>
    <metric>sum [same-ethnic-links] of turtles with [item 2 traits = "y"]</metric>
    <metric>sum [same-ethnic-links] of turtles with [item 2 traits = "z"]</metric>
    <enumeratedValueSet variable="new-link-chance">
      <value value="0"/>
      <value value="0.1"/>
      <value value="0.2"/>
      <value value="0.3"/>
      <value value="0.4"/>
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="eth-proportions">
      <value value="&quot;[0.75 0.2 0.028 0.022]&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ticks-per-year">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="sd-education-pref">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mean-dating">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num-agents">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ethnicity-x">
      <value value="&quot;Other White&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ethnicity-y">
      <value value="&quot;Indian&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="second-chance-interval">
      <value value="6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="sd-dating">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ethnicity-w">
      <value value="&quot;White British&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ethnicity-z">
      <value value="&quot;Pakistani&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="love-radar">
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="update-threshold">
      <value value="5"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="group size" repetitions="10" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="120"/>
    <metric>item 0 array:to-list num-turtles</metric>
    <metric>item 1 array:to-list num-turtles</metric>
    <metric>item 2 array:to-list num-turtles</metric>
    <metric>item 3 array:to-list num-turtles</metric>
    <metric>item 0 array:to-list married-turtles</metric>
    <metric>item 1 array:to-list married-turtles</metric>
    <metric>item 2 array:to-list married-turtles</metric>
    <metric>item 3 array:to-list married-turtles</metric>
    <metric>item 0 array:to-list out-percent</metric>
    <metric>item 1 array:to-list out-percent</metric>
    <metric>item 2 array:to-list out-percent</metric>
    <metric>item 3 array:to-list out-percent</metric>
    <metric>cross-ethnic</metric>
    <enumeratedValueSet variable="eth-proportions">
      <value value="&quot;[0.906 0.016 0.028 0.05]&quot;"/>
      <value value="&quot;[0.856 0.016 0.028 0.1]&quot;"/>
      <value value="&quot;[0.806 0.016 0.028 0.15]&quot;"/>
      <value value="&quot;[0.756 0.016 0.028 0.2]&quot;"/>
      <value value="&quot;[0.706 0.016 0.028 0.25]&quot;"/>
      <value value="&quot;[0.656 0.016 0.028 0.3]&quot;"/>
      <value value="&quot;[0.606 0.016 0.028 0.35]&quot;"/>
      <value value="&quot;[0.556 0.016 0.028 0.4]&quot;"/>
      <value value="&quot;[0.506 0.016 0.028 0.45]&quot;"/>
      <value value="&quot;[0.456 0.016 0.028 0.5]&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ethnicity-w">
      <value value="&quot;White British&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ethnicity-x">
      <value value="&quot;Other White&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ethnicity-y">
      <value value="&quot;Indian&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ethnicity-z">
      <value value="&quot;Pakistani&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="love-radar">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="update-threshold">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="second-chance-interval">
      <value value="6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="sd-dating">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ticks-per-year">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num-agents">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="sd-education-pref">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mean-dating">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="new-link-chance">
      <value value="0.5"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="group size addition" repetitions="2" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="120"/>
    <metric>item 0 array:to-list num-turtles</metric>
    <metric>item 1 array:to-list num-turtles</metric>
    <metric>item 2 array:to-list num-turtles</metric>
    <metric>item 3 array:to-list num-turtles</metric>
    <metric>item 0 array:to-list married-turtles</metric>
    <metric>item 1 array:to-list married-turtles</metric>
    <metric>item 2 array:to-list married-turtles</metric>
    <metric>item 3 array:to-list married-turtles</metric>
    <metric>item 0 array:to-list out-percent</metric>
    <metric>item 1 array:to-list out-percent</metric>
    <metric>item 2 array:to-list out-percent</metric>
    <metric>item 3 array:to-list out-percent</metric>
    <metric>cross-ethnic</metric>
    <enumeratedValueSet variable="eth-proportions">
      <value value="&quot;[0.906 0.016 0.028 0.05]&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ethnicity-w">
      <value value="&quot;White British&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ethnicity-x">
      <value value="&quot;Other White&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ethnicity-y">
      <value value="&quot;Indian&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ethnicity-z">
      <value value="&quot;Pakistani&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="love-radar">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="update-threshold">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="second-chance-interval">
      <value value="6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="sd-dating">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ticks-per-year">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num-agents">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="sd-education-pref">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mean-dating">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="new-link-chance">
      <value value="0.5"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
1
@#$#@#$#@
